//
//  CircleCounterView.m
//  timerCount
//
//  Created by Hans De Smedt on 19/02/13.
//  Copyright (c) 2013 Hans De Smedt. All rights reserved.
//

#import "CircleCounterView.h"

#define kCircleSegs 100

@interface CircleCounterView ()
@property (nonatomic, retain) NSString *timeFormatString;

- (void)setup;
- (void)update:(id)sender;
- (void)updateTime:(id)sender;

@end

@implementation CircleCounterView

@synthesize numberColor = mNumberColor;
@synthesize numberFont = mNumberFont;
@synthesize circleColor = mCircleColor;
@synthesize circleBorderWidth = mCircleBorderWidth;
@synthesize timeFormatString = mTimeFormatString;
@synthesize circleIncre = mCircleIncre;
@synthesize circleTimeInterval = mCircleTimeInterval;
@synthesize delegate = mDelegate;

- (void)awakeFromNib {
    [super awakeFromNib];
    [self setup];
}

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        [self setup];
        [self setBackgroundColor:[UIColor clearColor]];
    }
    return self;
}


// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect
{
    CGContextRef context = UIGraphicsGetCurrentContext();
    
    CGContextClearRect(context, rect);
    
    float radius = CGRectGetWidth(rect)/2.0f - self.circleBorderWidth/2.0f;
    float angleOffset = M_PI_2;
    
    CGContextBeginPath(context);
    UIBezierPath *aPath = [UIBezierPath bezierPathWithArcCenter:CGPointMake(CGRectGetMidX(rect), CGRectGetMidY(rect))
                                                         radius:radius
                                                     startAngle:-angleOffset
                                                       endAngle:(mCircleSegs + 1)/(float)kCircleSegs*M_PI*2 - angleOffset
                                                      clockwise:YES];
    
    CGPathRef shape = CGPathCreateCopyByStrokingPath(aPath.CGPath, NULL, self.circleBorderWidth, kCGLineCapSquare, kCGLineJoinMiter, 1.0f);
    CGContextAddPath(context, shape);
    CGContextClip(context);
    
    CGContextTranslateCTM(context, 0.5f * rect.size.width, 0.5f * rect.size.height );
    CGContextRotateCTM(context, (mCircleSegs + 2)/(float)kCircleSegs*M_PI*2 - M_PI);
    CGContextTranslateCTM(context, -0.5f * rect.size.width, -0.5f * rect.size.height );
    CGContextDrawImage(context, rect,background.CGImage);
}


- (void)setup {
    
    mIsRunning = NO;
    
    //< Default Parameters
    self.numberColor = [UIColor blackColor];
    self.numberFont = [UIFont fontWithName:@"Courier-Bold" size:20.0f];
    self.circleColor = [UIColor blackColor];
    self.circleBorderWidth = 20;
    self.timeFormatString = @"%.0f";
    self.circleIncre = NO;
    self.circleTimeInterval = 1.0f;
    
    mTimeInSeconds = 20;
    mTimeInterval = 1;
    mCircleSegs = 0;
    
    background = [UIImage imageNamed:@"background.png"];
}

#pragma mark - Public Methods
- (void)startWithSeconds:(float)seconds andInterval:(float)interval andTimeFormat:(NSString *)timeFormat {
    self.timeFormatString = timeFormat;
    [self startWithSeconds:seconds andInterval:interval];
}

- (void)startWithSeconds:(float)seconds andInterval:(float)interval {
    if (interval > seconds)
    {
        mTimeInterval = seconds/10.0f;
    }
    else
    {
        mTimeInterval = interval;
    }
    [self startWithSeconds:seconds];
}

- (void)startWithSeconds:(float)seconds {
    if (seconds > 0)
    {
        mTimeInSeconds = seconds;
        mIsRunning = YES;
        mCircleSegs = 0;
        [self update:nil];
        [self updateTime:nil];
    }
}

- (void)stop {
    mIsRunning = NO;
}

#pragma mark - Private Methods
- (void)update:(id)sender {
    if (mIsRunning)
    {
        mCircleSegs = (mCircleSegs + 1) % kCircleSegs;
        if (fabs(mTimeInSeconds) < 1e-4)
        {
            //< Finished
            mCircleSegs = (kCircleSegs - 1);
            mTimeInSeconds = 0;
            [self.delegate counterDownFinished:self];
        }
        else
        {
            [NSTimer scheduledTimerWithTimeInterval:self.circleTimeInterval/kCircleSegs
                                             target:self
                                           selector:@selector(update:)
                                           userInfo:nil
                                            repeats:NO];
        }
        [self setNeedsDisplay];
    }
}

- (void)updateTime:(id)sender {
    if (mIsRunning)
    {
        mTimeInSeconds -= mTimeInterval;
        if (fabs(mTimeInSeconds) < 1e-4)
        {
            //< Finished
            mCircleSegs = (kCircleSegs - 1);
            mTimeInSeconds = 0;
            [self.delegate counterDownFinished:self];
        }
        else
        {
            [NSTimer scheduledTimerWithTimeInterval:mTimeInterval
                                             target:self
                                           selector:@selector(updateTime:)
                                           userInfo:nil
                                            repeats:NO];
        }
    }
}

@end
